from datetime import datetime


class ColorPrinter:
    RED = "\033[31m"
    YELLOW = "\033[33m"
    BLUE = "\033[34m"
    GREEN = "\033[32m"
    CYAN = "\033[36m"
    GRAY = "\033[90m"
    MAGENTA = "\033[35m"
    BLACK = "\033[30m"
    WHITE = "\033[37m"
    RESET = "\033[0m"

    def output(self, text: str, mode: str):
        print(f"{mode}{text}{self.RESET}")


class Printer(ColorPrinter):
    def now(self):
        return datetime.now().strftime("%Y-%m-%d %H:%M:%S")

    def log(self, text: str, mode: str):
        print(f"{mode}{self.now()}  {text}{self.RESET}")

    def add_time(self, text: str):
        return "{}  {}".format(self.now(), text)

    def red(self, text: str):
        self.output(self.add_time(text), self.RED)

    def yellow(self, text: str):
        self.output(self.add_time(text), self.YELLOW)

    def blue(self, text: str):
        self.output(self.add_time(text), self.BLUE)

    def green(self, text: str):
        self.output(self.add_time(text), self.GREEN)

    def cyan(self, text: str):
        self.output(self.add_time(text), self.CYAN)

    def gray(self, text: str):
        self.output(self.add_time(text), self.GRAY)

    def magenta(self, text: str):
        self.output(self.add_time(text), self.MAGENTA)

    def black(self, text: str):
        self.output(self.add_time(text), self.BLACK)

    def white(self, text: str):
        self.output(self.add_time(text), self.WHITE)


if __name__ == '__main__':
    p = Printer()

    p.output("警告输出", p.YELLOW)
    p.output("错误输出", p.RED)

    p.red("红色")
    p.yellow("黄色")
    p.blue("蓝色")
    p.green("绿色")



class DictConvertObject:
    """
    dict 转换为对象
    object.name 等同于 dict["name"]
    """

    def __init__(self, src: dict):
        self._src = src

    def __getattr__(self, key):
        if key in self._src:
            value = self._src[key]
            if isinstance(value, dict):
                return DictConvertObject(value)
            elif isinstance(value, list):
                return [DictConvertObject(v) if isinstance(v, dict) else v for v in value]
            return value
        else:
            raise AttributeError(f"'{self.__class__.__name__}' object has no attribute '{key}'")


if __name__ == '__main__':
    src = {
        "data": {
            "list": [
                {
                    "name": "Thomas",
                    "age": 55,
                    "other": [
                        {
                            "mark": "记号"
                        }
                    ]
                },
                {
                    "name": "Mark",
                    "age": 22
                }
            ]
        },
        "status": 0,
        "code": 200
    }
    data = DictConvertObject(src)
    print(data.data.list[1].name)
    print(data.data.list[0].other[0].mark)
    print(data.status)
    print(data.code)



import re


class Cookier:
    def __init__(self, cookie: str | dict):
        assert isinstance(cookie, (str, dict)), "Cookie must be str or dict"
        if isinstance(cookie, str):
            self.cookie_str = cookie
            self.cookie_dict = self.cookie_to_dict(cookie)
        else:
            self.cookie_str = self.cookie_to_str(cookie)
            self.cookie_dict = cookie

    @staticmethod
    def cookie_to_str(cookie: dict) -> str:
        """Cookie 转换为 str 类型"""
        cookie_str = ""
        for key, value in cookie.items():
            cookie_str += "{}={}; ".format(key, value)
        return cookie_str.rstrip("; ")

    @staticmethod
    def cookie_to_dict(cookie: str) -> dict:
        """Cookie 转换为 dict 类型"""
        cookie = cookie.rstrip().rstrip(";")
        cookie_dict = {kv.split("=")[0]: kv.split("=")[1] for kv in cookie.split("; ")}
        return cookie_dict

    def get_value(self, key: str) -> str:
        """获取 Cookie 中某个字段的值"""
        rule = key + "=([^;]+)"
        match = re.search(rule, self.cookie_str)
        return match.group(1)

    def remove_value(self, key: str) -> str:
        """删除 Cookie 中某个字段，然后返回新的 Cookie"""
        self.cookie_dict.pop(key)
        self.cookie_str = self.cookie_to_str(self.cookie_dict)
        return self.cookie_str

    def list_keys(self) -> list[str]:
        return list(self.cookie_dict.keys())


if __name__ == '__main__':
    cookie = "a=1; b=2; c=3; d=4; e=5"
    c = Cookier(cookie)

    keys = c.list_keys()
    print(keys)

    a = c.get_value("a")
    print(a)

    c.remove_value("c")
    print(c.cookie_str)
    print(c.cookie_dict)



import time
from datetime import datetime, timedelta


class TimeConvert:
    @staticmethod
    def ts_to_time(ts: float) -> str:
        """时间戳转时间"""
        date_fmt = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(ts))
        return date_fmt

    @staticmethod
    def time_to_ts(date_formatted: str) -> int:
        """时间转时间戳"""
        ts = time.mktime(time.strptime(date_formatted, "%Y-%m-%d %H:%M:%S"))
        return int(ts)

    @staticmethod
    def today_anytime_ts(hour: int, minute: int, second=0) -> float:
        """获取今天任意时刻的时间戳"""
        now = datetime.now()
        today_0 = now - timedelta(hours=now.hour, minutes=now.minute, seconds=now.second)
        today_anytime = today_0 + timedelta(hours=hour, minutes=minute, seconds=second)
        ts = today_anytime.timestamp()
        return ts

    @staticmethod
    def now():
        return datetime.now().strftime("%Y-%m-%d %H:%M:%S")


class Timer:
    def __init__(self, desc: str):
        self.desc = desc

    def __enter__(self):
        self.start_time = time.time()
        return self

    def __exit__(self, *args):
        self.end_time = time.time()
        print(f"{self.desc}: cons {self.end_time - self.start_time:.2f} sec")


if __name__ == '__main__':
    t = TimeConvert()
    ts = t.today_anytime_ts(21, 22, 23)
    print("时间戳转时间：{}".format(t.ts_to_time(ts)))
    print("时间转时间戳 {}".format(t.time_to_ts("2025-01-01 12:00:00")))
    print("现在的时间是：{}".format(t.now()))

    with Timer("计时器-1"):
        lis = [i for i in range(1000000)]

    with Timer("计时器-2"):
        lis2 = []
        for i in range(1000000):
            lis2.append(i)
